from .utils import *


def bot(message, user_id, platform, channel_id):
    try:
        message = message.strip()

        # help
        if config.features.get('help', True):
            if message.startswith('帮助'):
                return help_command()
            if message.startswith('help'):
                arg_text = message[4:].strip()
                return help_command(arg_text)
            if message.endswith('-h'):
                arg_text = message[:-2].strip()
                return help_command(arg_text)

        # 进行车牌匹配
        if config.features.get('car_number_forwarding'):
            status = submit_car_number_msg(message, user_id, platform)
            if status:
                return None  # 已经匹配了车牌，就不需要再匹配其他指令了

        # 进行 v2 api 命令匹配
        command_matched, api = match_command(message, load_commands_from_config(config.commands))
        if command_matched:
            return v2_api_command(message, command_matched, api, platform, user_id, channel_id)
        if config.user_database_path:
            return bot_extra_local_database(message, user_id, platform)
        return bot_extra_remote_server(message, user_id, platform)
    except Exception as e:
        logger.error(f'Error: {e}')
        # raise e
        return []


def bot_extra_local_database(message, user_id, platform):
    # 玩家状态
    if config.features.get('player_status', True):
        if message.endswith('服玩家状态'):
            return player_status(user_id, platform, message[:-4]) if server_exists(r_ := query_server_info(message[:-4])) else text_response('未找到被指定的服务器') if len(message) <= 7 else None

        if message.startswith('玩家状态'):
            if (arg_ := message[4:].strip()) == '':
                return player_status(user_id, platform)
            # 如果用户输入了数字
            if arg_.isdigit():
                return player_status(user_id, platform, int(arg_))
            # 如果用户输入了服务器名
            if server_exists(r_ := query_server_info(arg_)):
                return player_status(user_id, platform, arg_)
            else:
                return text_response('未找到绑定记录')
    # 绑定玩家
    if config.features.get('bind_player', True):
        if message.startswith('绑定玩家'):
            if server_exists(query_server_info(message[4:].strip())) or message[4:].strip() == '':
                return bind_player_request(platform, user_id)
            else:
                return text_response(f'未找到名为 {(message[4:]).strip()} 的服务器信息，请确保输入的是服务器名而不是玩家ID，通常情况发送"绑定玩家"即可')

        if message.startswith('验证'):
            arg = message.replace('验证', '').strip()
            # 正则出数字
            player_ids = re.findall(r'\d+', arg)
            if not player_ids or len(player_ids) > 1:
                return text_response('请确保输入正确(例如: 验证 10000xxxxx cn)')
            player_id = player_ids[0]
            server = query_server_info(arg.replace(player_id, ''))  # 后续自动处理 None
            return bind_player_verification(platform, user_id, server, player_id, True)

        if message.startswith('解除绑定'):
            if message[4:].strip() == '':
                return unbind_player_request(platform, user_id)
            msg_list = message.split(' ')
            if len(msg_list) < 2:
                return unbind_player_request(platform, user_id)
            if not msg_list[-1].isdigit():
                return unbind_player_request(platform, user_id)
            record = int(msg_list[-1])
            if record == 0:
                return text_response('为什么会有0啊，服了')
            return unbind_player_verification(platform, user_id, record)

    # 车牌转发控制
    if config.features.get('switch_car_forwarding', True):
        if message in ['开启车牌转发', '开启个人车牌转发']:
            return set_car_forward(platform, user_id, True)

        if message in ['关闭车牌转发', '关闭个人车牌转发']:
            return set_car_forward(platform, user_id, False)

    # 切换服务器模式
    if config.features.get('change_main_server', True):
        if message.endswith('服模式'):
            return set_server_mode(platform, user_id, message[:-2])

        if message.startswith('主服务器'):
            return set_server_mode(platform, user_id, message[4:].strip())

    # 设置默认服务器列表
    if config.features.get('change_server_list', True):
        if message.startswith('设置默认服务器'):
            return set_default_server(platform, user_id, message[6:].strip())

    return None


def bot_extra_remote_server(message, user_id, platform):
    # 玩家状态
    if config.features.get('player_status', True):
        if message.endswith('服玩家状态'):
            # 如果匹配 *服玩家状态 则查询对应服务器的玩家状态 前提是用户输入的 * 是一个有效的服务器名 否则返回None
            return Remote.player_status(user_id, platform, r_) if server_exists(r_ := query_server_info(message[:-4])) else text_response('未找到服务器') if len(message) <= 7 else None

        if message.startswith('玩家状态'):
            # 如果匹配 玩家状态 则查询默认服务器的玩家状态 如果用户输入了服务器名 则查询对应服务器的玩家状态，如果服务器名无效则返回None
            return Remote.player_status(user_id, platform) if (arg_ := message[4:].strip()) == '' else (Remote.player_status(user_id, platform, r_) if server_exists(r_ := query_server_info(arg_)) else None)

    # 绑定玩家
    if config.features.get('bind_player', True):
        if message.startswith('绑定玩家'):
            # 如果匹配 绑定玩家 则绑定默认服务器的玩家 如果用户输入了服务器名 则绑定对应服务器的玩家，如果服务器名无效则 赋值为 None
            server = Remote.get_user_data(platform, user_id)['data']['server_mode'] if message[4:].strip() == '' else (r_ if server_exists(r_ := query_server_info(message[4:])) else None)
            if not server_exists(server):
                return text_response(f'未找到名为 {(message[4:]).strip()} 的服务器信息，请确保输入的是服务器名而不是玩家ID，通常情况发送"绑定玩家"即可')

            res = Remote.bind_player_request(platform, user_id, server, True)
            if res.get('status') != 'success':
                # {'status': 'success', 'data': {'verifyCode': 12492}}
                return text_response(res.get('data'))
            # if not res.get('status') == 'failed':
            #     return text_response('未知错误喵')
            return text_response(f'''正在绑定账号，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{res.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证 + 空格 + 玩家ID 来完成本次身份验证\n验证 10000xxxx 国服''')

        if message.startswith('解除绑定'):
            server = Remote.get_user_data(platform, user_id)['data']['server_mode'] if message[4:].strip() == '' else (r_ if (config.server_list(r_ := query_server_info(message[4:]))) else None)
            if server_exists(server) is False:
                return text_response(f'未找到名为 {(message[4:]).strip()} 的服务器信息，请确保输入的是服务器名而不是玩家ID，通常情况发送"绑定玩家"即可')
            response = Remote.bind_player_request(platform, user_id, server, False)  # 获取响应
            if response.get('status') == 'failed':  # 检查状态
                return text_response(response.get('data'))
            # 如果是200
            return text_response(f'''正在解除，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{response.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证解绑 {server}\n来完成本次身份验证(没错只需要加上 {server} 来确定需要解绑的服务器)''')

        if message.startswith('验证解绑'):
            if not message[4:].strip().isdigit():
                return text_response('请输入解绑时提供的serverID(数字)')
            r = Remote.bind_player_verification(platform, user_id, int(message[4:].strip()), Remote.get_user_data(platform, user_id)['data']['server_list'][int(message[4:].strip())]['playerId'], False)
            if r.get('status') == 'failed':
                return text_response(r.get('data'))
            return text_response('解绑成功')

        if message.startswith('验证'):
            if not message[2:].strip().isdigit():
                return text_response('请输入正确的玩家ID(数字)')
            if r := Remote.bind_player_verification(platform, user_id, Remote.get_user_data(platform, user_id)['data']['server_mode'], message[2:].strip(), True).get('status') == 'failed':
                return text_response(r.get('data')) if r.get('data') != '错误: 未请求绑定或解除绑定玩家' else None
            return text_response('绑定成功！现在可以使用"玩家状态"来查询玩家信息')

    # 车牌转发控制
    if config.features.get('switch_car_forwarding', True):
        if message in ['开启车牌转发', '开启个人车牌转发']:
            r = Remote.set_car_forward(platform, user_id, True)
            return text_response('已开启车牌转发') if r.get('status') == 'success' else None

        if message in ['关闭车牌转发', '关闭个人车牌转发']:
            r = Remote.set_car_forward(platform, user_id, False)
            return text_response('已关闭车牌转发') if r.get('status') == 'success' else None

    # 切换服务器模式
    if config.features.get('change_main_server', True):
        if message.endswith('服模式'):
            if r := Remote.set_server_mode(platform, user_id, message[:-2]).get('status') == 'success':
                return text_response('已切换为' + message[:-2] + '模式')
            return text_response(r.get('data')) if len(message) <= 5 else None

        if message.startswith('主服务器'):
            if r := Remote.set_server_mode(platform, user_id, message[4:].strip()).get('status') == 'success':
                return text_response('主服务器已切换为' + message[4:].strip())
            return text_response(r.get('data')) if ' ' in message else None

    # 设置默认服务器列表
    if config.features.get('change_server_list', True):
        if message.startswith('设置默认服务器'):
            if r := Remote.set_default_server(platform, user_id, message[6:].strip()).get('status') == 'success':
                return text_response('默认服务器已设置为' + message[6:].strip())
            return text_response(r.get('data')) if ' ' in message else None

    return None





from typing import List
from .utils import *


class Router:
    def __init__(self):
        pass

    @staticmethod
    def help(text: str, default_servers: List[int], server: int):
        return help_command(text)

    @staticmethod
    def player(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('player', text, default_servers, server)

    @staticmethod
    def card(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('card', text, default_servers, server)

    @staticmethod
    def gacha(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('gacha', text, default_servers, server)

    @staticmethod
    def gachaSimulate(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('gachaSimulate', text, default_servers, server)

    @staticmethod
    def event(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('event', text, default_servers, server)

    @staticmethod
    def song(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('song', text, default_servers, server)

    @staticmethod
    def song_meta(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('songMeta', text, default_servers, server)

    @staticmethod
    def character(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('character', text, default_servers, server)

    @staticmethod
    def chart(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('chart', text, default_servers, server)

    @staticmethod
    def ycxall(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('ycxAll', text, default_servers, server)

    @staticmethod
    def ycx(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('ycx', text, default_servers, server)

    @staticmethod
    def lsycx(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('lsycx', text, default_servers, server)

    @staticmethod
    def ycm(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('ycm', text, default_servers, server)

    @staticmethod
    def card_illustration(text: str, default_servers: List[int], server: int):
        return v2api_from_backend('cardIllustration', text, default_servers, server)

    @staticmethod
    def bind_player_request(platform: str, user_id: str):
        return bind_player_request(platform, user_id)

    @staticmethod
    def bind_player_verification(platform: str, user_id: str, server: int, player_id: str, status: bool):
        return bind_player_verification(platform, user_id, server, player_id, status)

    @staticmethod
    def set_car_forward(platform: str, user_id: str, status: bool):
        return set_car_forward(platform, user_id, status)

    @staticmethod
    def set_default_server(platform: str, user_id: str, text: str):
        return set_default_server(platform, user_id, text)

    @staticmethod
    def set_server_mode(platform: str, user_id: str, text: str):
        return set_server_mode(platform, user_id, text)

    @staticmethod
    def get_user_data(platform: str, user_id: str):
        return get_user_data(platform, user_id)


router = Router()


class InteriorLocal:
    def __init__(self):
        pass

    @staticmethod
    def bind_player_request(platform: str, user_id: str):
        return bind_player_request(platform, user_id)

    @staticmethod
    def bind_player_verification(platform: str, user_id: str, server: int, player_id: str, status: bool):
        return bind_player_verification(platform, user_id, server, player_id, status)

    @staticmethod
    def set_car_forward(platform: str, user_id: str, status: bool):
        return set_car_forward(platform, user_id, status)

    @staticmethod
    def set_default_server(platform: str, user_id: str, text: str):
        return set_default_server(platform, user_id, text)

    @staticmethod
    def set_server_mode(platform: str, user_id: str, text: str):
        return set_server_mode(platform, user_id, text)

    @staticmethod
    def get_user_data(platform: str, user_id: str):
        return get_user_data(platform, user_id)

    @staticmethod
    def submit_car_number_msg(message: str, user_id: str, platform: str | None = None):
        return submit_car_number_msg(message, user_id, platform)


interior_local_method = InteriorLocal()


class InteriorRemote:
    def __init__(self):
        pass

    @staticmethod
    def bind_player_request(platform: str, user_id: str):
        return Remote.bind_player_request(platform, user_id)

    @staticmethod
    def bind_player_verification(platform: str, user_id: str, server: int, player_id: str, status: bool):
        return Remote.bind_player_verification(platform, user_id, server, player_id, status)

    @staticmethod
    def set_car_forward(platform: str, user_id: str, status: bool):
        return Remote.set_car_forward(platform, user_id, status)

    @staticmethod
    def set_default_server(platform: str, user_id: str, text: str):
        return Remote.set_default_server(platform, user_id, text)

    @staticmethod
    def set_server_mode(platform: str, user_id: str, text: str):
        return Remote.set_server_mode(platform, user_id, text)

    @staticmethod
    def get_user_data(platform: str, user_id: str):
        return Remote.get_user_data(platform, user_id)

    @staticmethod
    def submit_car_number_msg(message: str, user_id: str, platform: str | None = None):
        return submit_car_number_msg(message, user_id, platform)


interior_remote_method = InteriorRemote()


from typing import List
import re
import json
import sqlite3
import os
import sys
import random
import urllib3
from urllib3.exceptions import HTTPError
from loguru import logger

from .config import config


class DatabaseManager:
    def __init__(self, path):
        self.path = path
        self.conn = None
        self.cursor = None
        self.init_db(self.path)

    def init_db(self, path):
        if path:
            self.conn = sqlite3.connect(path, check_same_thread=False)
            logger.success(f'数据库连接成功，路径: {path}')
            self.cursor = self.conn.cursor()
            self.cursor.execute('''
                CREATE TABLE IF NOT EXISTS users (
                    id INTEGER PRIMARY KEY AUTOINCREMENT,
                    user_id TEXT UNIQUE,
                    platform TEXT,
                    server_mode INTEGER,
                    default_server TEXT,
                    car INTEGER,
                    game_ids TEXT,
                    verify_code TEXT
                )
            ''')
            self.conn.commit()
        else:
            # 如果不使用数据库，设置连接为 None
            self.conn = None
            # logger.info('不使用数据库')

    def close_db(self):
        if self.conn:
            self.conn.close()

    def execute_query(self, query):
        if self.cursor:
            self.cursor.execute(query)
            return self.cursor.fetchall()
        else:
            return []


db_manager = DatabaseManager(config.user_database_path)


def database(path: str | None = None):
    '''
    初始化数据库
    :return:
    '''
    if not path:
        path = (os.path.dirname(sys.modules['__main__'].__file__))
    config.user_database_path = path
    db_manager.init_db(path)


def text_response(string):
    return [{"type": "string", "string": string}]


def convert_server_names_to_indices(server_names: str) -> list:
    indices_list = [config.server_name_to_index.get(name, "Unknown") for name in server_names.split(" ")]
    result = [index for index in indices_list if index != "Unknown"]
    # 转化成数字，例如 ["0", "1"] -> [0, 1]
    return [int(i) for i in result]


def query_server_info(server_name: str) -> int:
    server = convert_server_names_to_indices(server_name)[0] if convert_server_names_to_indices(server_name) else None
    return server


def server_exists(server):
    if server or server == 0:
        return True
    return False


def requests_post_for_user(url, data):
    if config.user_data_backend_use_proxy:
        http = urllib3.ProxyManager(config.proxy_url, cert_reqs='CERT_NONE')
    else:
        http = urllib3.PoolManager(cert_reqs='CERT_NONE')
    try:
        response = http.request('POST', url, headers={'Content-Type': 'application/json'}, body=json.dumps(data))

        # 检查响应的状态码是否为 200
        if response.status == 200:
            return json.loads(response.data.decode('utf-8'))
        else:
            # 处理其他状态码
            return text_response("服务器出现了问题，请稍后再试。")

    except HTTPError as http_err:
        logger.error(f'HTTP 错误：{http_err}')
        return text_response("服务器出现了问题，请稍后再试。")

    except Exception as e:
        logger.error(f'发生异常：{e}')
        return text_response("发生了未知错误。")


def requests_post_for_backend(url, data):
    if config.backend_use_proxy:
        http = urllib3.ProxyManager(config.proxy_url, cert_reqs='CERT_NONE')
    else:
        http = urllib3.PoolManager(cert_reqs='CERT_NONE')
    try:
        response = http.request('POST', url, headers={'Content-Type': 'application/json'}, body=json.dumps(data))

        # 检查响应的状态码是否为 200
        if response.status == 200:
            return json.loads(response.data.decode('utf-8'))
        else:
            # 处理其他状态码
            return text_response("服务器出现了问题，请稍后再试。")

    except HTTPError as http_err:
        logger.error(f'HTTP 错误：{http_err}')
        return text_response("服务器出现了问题，请稍后再试。")

    except Exception as e:
        logger.error(f'发生异常：{e}')
        return text_response("发生了未知错误。")


def v2api_from_backend(api, text, default_servers: List[int] = None, server=3):
    if default_servers is None:
        default_servers = [3, 0]
    path = f"/v2/{api}"
    data = {
        "default_servers": default_servers,
        "server": server,
        "text": text,
        "useEasyBG": config.use_easy_bg,
        "compress": config.compress
    }
    res = requests_post_for_backend(f"{config.backend}{path}", data)
    return res


def v2_api_command(message, command_matched, api, platform, user_id, channel_id):
    text = message[len(command_matched):].strip()

    if api in ['cardIllustration', 'ycm']:  # 无需验证server信息
        return v2api_from_backend(api, text)

    if api == 'gachaSimulate':
        if channel_id in config.ban_gacha_simulate_group_data:
            return text_response('此群禁止使用模拟抽卡功能')

    # 获取用户数据
    import logging
    logging.debug(config.user_database_path)
    user_data = get_user_data(platform, user_id) if config.user_database_path else Remote.get_user_data(platform, user_id)
    try:
        if user_data['status'] != 'success':
            return text_response('获取用户数据失败：内部错误')
        return v2api_from_backend(api, text, user_data['data']['default_server'], user_data['data']['server_mode'])
    except Exception as e:
        return text_response('获取用户数据失败：网络 / 前端错误')


def submit_car_number_msg(message, user_id, platform=None):
    # 检查car_config['car']中的关键字
    for keyword in config.car_config["car"]:
        if str(keyword) in message:
            break
    else:
        return False
    # 检查car_config['fake']中的关键字
    for keyword in config.car_config["fake"]:
        if str(keyword) in message:
            return False
    pattern = r"^\d{5}(\D|$)|^\d{6}(\D|$)"
    if not re.match(pattern, message):
        return False

    # 获取用户数据
    try:
        if platform:
            user_data = get_user_data(platform, user_id) if config.user_database_path else Remote.get_user_data(platform, user_id)
            if not user_data['data']['car']:
                return True
    except Exception as e:
        logger.error('unknown user')
        # 默认不开启关闭车牌，继续提交
        pass

    try:
        car_id = message[:6]
        if not car_id.isdigit() and car_id[:5].isdigit():
            car_id = car_id[:5]

        # 构建 URL
        url = f"https://api.bandoristation.com/index.php?function=submit_room_number&number={car_id}&user_id={user_id}&raw_message={message}&source={config.token_name}&token={config.bandori_station_token}"

        if config.submit_car_number_use_proxy:
            http = urllib3.ProxyManager(config.proxy_url, cert_reqs='CERT_NONE')
        else:
            http = urllib3.PoolManager(cert_reqs='CERT_NONE')

        # 发送请求
        response = http.request('GET', url)

        # 检查响应的状态码是否为 200
        if response.status == 200:
            return True
        else:
            logger.error(f"[Tsugu] 提交车牌失败，HTTP响应码: {response.status}")
            return True  # 虽然提交失败，但是确定了是车牌消息

    except Exception as e:
        logger.error(f"[Tsugu] 发生异常: {e}")
        return True  # 虽然提交失败，但是确定了是车牌消息


def match_command(message, cmd_dict):
    for command, api_value in cmd_dict.items():
        if message.startswith(command):
            return command, api_value
    return None, None


def load_commands_from_config(data):
    # 初始化一个空字典来存储命令到操作的映射
    cmd_dict = {}
    for item in data:
        api = item['api']
        for command_name in item['command_name']:
            cmd_dict[command_name] = api
    return cmd_dict



def get_user_data(platform: str, user_id: str):

    cursor = db_manager.conn.cursor()
    cursor.execute("SELECT * FROM users WHERE user_id = ? AND platform = ?", (user_id, platform))
    row = cursor.fetchone()

    if row:
        data = {
            "user_id": row[1],
            "platform": row[2],
            "server_mode": row[3],
            "default_server": list(json.loads(row[4])),
            "car": row[5],
            "game_ids": row[6],
            "verify_code": row[7]
        }
    else:
        data = {
            "user_id": user_id,
            "platform": platform,
            "server_mode": 3,
            "default_server": [3, 0],
            "car": 1,
            "game_ids": json.dumps([]),
            "verify_code": ""
        }
        cursor.execute('''
        INSERT INTO users (user_id, platform, server_mode, default_server, car, game_ids)
        VALUES (?, ?, ?, ?, ?, ?)
        ''', (user_id, platform, 3, json.dumps([3, 0]), 1, json.dumps([])))
        db_manager.conn.commit()
    result = {
        "status": "success",
        "data": data
    }
    return result


def player_status(user_id, platform, args: str | int | None = None):
    user_data = get_user_data(platform, user_id)
    game_ids = user_data['data']['game_ids']
    if game_ids == "[]" or not game_ids:
        return text_response(f'未绑定玩家，请发送 绑定玩家 进行绑定')
    game_ids = json.loads(game_ids)
    text = ''
    if args is None:
        # 先查找默认服务器对应的记录
        server: int = user_data['data']['server_mode']
        for i in game_ids:
            if i.get("server") == server:
                player_id = str(i.get("game_id"))
                server = int(i.get("server"))

                text = f'已查找默认服务器 {config.server_index_to_name[str(server)]} 的记录'
                break
        else:
            # 再查找第一个记录
            if game_ids:
                return text_response(f'未在 {len(game_ids)} 条记录中找到 {config.server_index_to_name[str(server)]} 的记录')
            else:
                pass  # 前面已经判断过了没绑定任何的情况
    elif isinstance(args, int):
        # 查找对应数字的记录
        if args == 0:
            return text_response('哪来的0（')
        if args > len(game_ids) or args < 1:
            return text_response(f'总共绑定了 {len(game_ids)} 条记录')
        player_id, server = str(game_ids[args - 1].get("game_id")), game_ids[args - 1].get("server")
        text = f'已为查找第{args}条记录'
        if not player_id:
            return text_response(f'未找到此服务器的记录，请检查是否绑定过此服务器')

    elif isinstance(args, str):
        server = query_server_info(args)
        # 查找对应服务器的记录
        for i in game_ids:
            if i.get("server") == server:
                player_id = str(i.get("game_id"))
                text = f'已查找服务器 {config.server_index_to_name[str(server)]} 的记录'
                break
        else:
            return text_response(f'未找到记录，请检查是否绑定过此服务器')
    else:
        return text_response('参数错误，的服务器可能不合法')
    result: list = v2api_from_backend('player', player_id, server=server)
    new_item = {"type": "string", "string": text}
    result.append(new_item)
    return result


def set_car_forward(platform, user_id, status: bool):

    if status:
        status = 1
    else:
        status = 0
    get_user_data(platform, user_id)
    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET car = ? WHERE user_id = ? AND platform = ?", (status, user_id, platform))
    db_manager.conn.commit()
    return text_response('车牌转发设置成功')


def set_default_server(platform, user_id, text):

    default_server = convert_server_names_to_indices(text)
    get_user_data(platform, user_id)
    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET default_server = ? WHERE user_id = ? AND platform = ?",
                   (json.dumps(default_server), user_id, platform))
    db_manager.conn.commit()
    return text_response(f'默认服务器设置为 {text}')


def set_server_mode(platform, user_id, text):

    server = int(convert_server_names_to_indices(text)[0]) if convert_server_names_to_indices(text) else None
    if server is None:
        return text_response(f'未找到名为 {text} 的服务器信息')
    get_user_data(platform, user_id)
    cursor = db_manager.conn.cursor()
    cursor.execute("UPDATE users SET server_mode = ? WHERE user_id = ? AND platform = ?",
                   (server, user_id, platform))
    db_manager.conn.commit()
    return text_response(f'服务器模式设置为 {text}')


def bind_player_request(platform: str, user_id: str):

    cursor = db_manager.conn.cursor()
    data = get_user_data(platform, user_id)

    bind_count = len(json.loads(data['data']['game_ids']))

    def generate_verify_code():
        while True:
            verify_code = random.randint(10000, 99999)
            if '64' not in str(verify_code) and '89' not in str(verify_code):
                return verify_code
    # 不包含64和89的随机数

    verify_code = generate_verify_code()
    # verify_code = "000000"
    rpl_true = f'正在绑定第{bind_count + 1}个记录，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{verify_code}\n稍等片刻等待同步后，发送\n验证 玩家ID 来完成本次身份验证\n例如：验证 10000xxxx 国服'
    # 存入verify_code
    cursor.execute("UPDATE users SET verify_code = ? WHERE user_id = ? AND platform = ?", (verify_code, user_id, platform))
    db_manager.conn.commit()
    return text_response(rpl_true)


def unbind_player_request(platform: str, user_id: str):

     data = get_user_data(platform, user_id)
     record = json.loads(data['data']['game_ids'])
     if not record:
         return text_response('未找到绑定记录')

     return text_response(f'当前有 {len(record)} 个记录，发送"解除绑定 {len(record)}"来获解除第{len(record)}个记录，以此类推')


def bind_player_verification(platform: str, user_id: str, server: int | None, player_id: str, bind_type: bool):

    cursor = db_manager.conn.cursor()
    # 先检查verify_code是否正确
    # 使用 get
    user_data = get_user_data(platform, user_id)
    verify_code = user_data['data']['verify_code']

    if server is None:
        server = user_data['data']['server_mode']
    if verify_code == "" or not verify_code:
        return text_response('请先获取验证代码')
    # 检测重复性
    game_ids = json.loads(user_data['data']['game_ids'])
    for i in game_ids:
        if i.get("game_id") == player_id and i.get("server") == server:
            return text_response('请勿重复绑定')
    server_s_name = config.server_index_to_s_name[str(server)]

    # 构建 URL
    url = f'https://bestdori.com/api/player/{server_s_name}/{player_id}?mode=2'
    if config.verify_player_bind_use_proxy:
        http = urllib3.ProxyManager(config.proxy_url, cert_reqs='CERT_NONE')
    else:
        http = urllib3.PoolManager(cert_reqs='CERT_NONE')
    # 发送请求
    response = http.request('GET', url)
    # 检查响应的状态码是否为 200
    if response.status == 200:
        # 解析 JSON 响应数据
        data = json.loads(response.data.decode('utf-8'))
    else:
        logger.error(f"获取玩家数据失败，HTTP响应码: {response.status}")
        return None

    if data.get("data").get("profile") is None or data.get("profile") == {}:
        return text_response('玩家ID不存在，请检查输入，或服务器是否对应')
    introduction = data.get("data", {}).get("profile", {}).get("introduction")
    deck_name = data.get("data", {}).get("profile", {}).get("mainUserDeck", {}).get("deckName")
    if verify_code != introduction and verify_code != deck_name:
        return text_response('验证失败，签名或者乐队编队名称与验证代码不匹配喵，可以检查后再次尝试(无需重复发送绑定玩家)')
    # 验证成功
    game_ids = json.loads(user_data['data']['game_ids'])
    game_ids.append({"game_id": player_id, "server": server})
    cursor.execute("UPDATE users SET game_ids = ? WHERE user_id = ? AND platform = ?",
                   (json.dumps(game_ids), user_id, platform))  # 存入game_ids
    cursor.execute("UPDATE users SET verify_code = ? WHERE user_id = ? AND platform = ?",
                   ("", user_id, platform))  # 清空verify_code
    db_manager.conn.commit()
    return text_response('绑定成功！现在可以使用"玩家状态"来查询玩家信息了')


def unbind_player_verification(platform: str, user_id: str, record: int | None):
    user_data = get_user_data(platform, user_id)

    cursor = db_manager.conn.cursor()

    # 先检查verify_code是否正确
    cursor.execute("SELECT * FROM users WHERE user_id = ? AND platform = ?", (user_id, platform))
    row = cursor.fetchone()

    if row:
        # 用户存在，解析game_ids字段
        game_ids = json.loads(user_data['data']['game_ids'])  # Convert JSON string to list

        if record and 1 <= record <= len(game_ids):  # 如果record是一个有效的索引
            # 将record从1-based转换为0-based索引
            index = record - 1

            # 删除指定索引的记录
            del game_ids[index]

            # 将更新后的game_ids列表转换回JSON字符串
            updated_game_ids_json = json.dumps(game_ids)

            # 更新数据库中的game_ids字段
            cursor.execute("UPDATE users SET game_ids = ? WHERE user_id = ? AND platform = ?", (updated_game_ids_json, user_id, platform))
            db_manager.conn.commit()
            return text_response('解绑成功喵')
        return text_response(f'解绑失败，当前有 {record} 个记录，发送"解除绑定 {record}"来获解除第{record}个记录，以此类推')
    return text_response('解绑失败，请检查输入是否正确')


def help_command(command_name=None):
    if not command_name:
        # 读取 config.help_doc_dict 中的所有键
        command_list = list(config.help_doc_dict.keys())
        command_list.sort()
        return text_response(f'当前支持的命令有：\n{", ".join(command_list)}\n 请使用"help 命令名"来查看命令的详细帮助')
    else:
        # 读取 config.help_doc_dict 中的指定键
        return None


class Remote:
    @staticmethod
    def get_user_data(platform: str, user_id: str):
        data = {
            'platform': platform,
            'user_id': user_id
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/getUserData", data)
        return result

    @staticmethod
    def bind_player_request(platform: str, user_id: str, server: int, bind_type: bool):
        data = {
            'platform': platform,
            'user_id': user_id,
            'server': server,
            'bindType': bind_type  # 布尔，表示绑定还是解绑
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/bindPlayerRequest", data)
        return result

    @staticmethod
    def bind_player_verification(platform: str, user_id: str, server: int, player_id: str, bind_type: bool):
        data = {
            'platform': platform,
            'user_id': user_id,
            'server': server,
            'playerId': player_id,
            'bindType': bind_type  # 布尔，表示绑定还是解绑
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/bindPlayerVerification", data)
        return result

    @staticmethod
    def player_status(user_id, platform, server=None):
        user_data = Remote.get_user_data(platform, user_id)
        if user_data['status'] != 'success':
            return text_response('获取用户数据失败')
        if server is None:
            server = user_data['data']['server_mode']
        player_id = user_data['data']['server_list'][server]['playerId']
        if player_id == 0:
            return text_response(f'未绑定玩家，请使用 绑定玩家 进行绑定')
        return v2api_from_backend('player', str(player_id), server=server)

    @staticmethod
    def set_car_forward(platform, user_id, status):
        data = {
            'platform': platform,
            'user_id': user_id,
            'status': status
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/changeUserData/setCarForwarding", data)
        return result

    @staticmethod
    def set_default_server(platform, user_id, text):
        data = {
            'platform': platform,
            'user_id': user_id,
            'text': text
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/changeUserData/setDefaultServer", data)
        return result

    @staticmethod
    def set_server_mode(platform, user_id, text):
        data = {
            'platform': platform,
            'user_id': user_id,
            'text': text
        }
        result = requests_post_for_user(f"{config.user_data_backend}/user/changeUserData/setServerMode", data)
        return result





import tsugu
import unittest
import os


class TestTsugu(unittest.TestCase):

    def test_tsugu(self):
        def show_back_msg(data):
            import base64
            from PIL import Image
            import io

            if not data:
                return "[无反馈数据]"

            for item in data:
                if item["type"] == "string":
                    print(f"[文字信息]\n{item['string']}")
                elif item["type"] == "base64":
                    image_data = base64.b64decode(item["string"])
                    print(f"[图像大小: {len(image_data) / 1024:.2f}KB]")
                    Image.open(io.BytesIO(image_data)).show()
                else:
                    print(item)


        tsugu.database(os.path.join(os.path.dirname(__file__), 'user_data.db'))
        # tsugu.database('./user_data.db')

        show_back_msg(tsugu.bot('玩家状态', '114514', 'red', '666808414'))


if __name__ == '__main__':
    unittest.main()





import json


class Config:
    def __init__(self):
        # 设置默认值
        self.backend = "http://tsugubot.com:8080"
        self.user_data_backend = "http://tsugubot.com:8080"
        self.backend_use_proxy = False
        self.user_data_backend_use_proxy = False
        self.submit_car_number_use_proxy = False
        self.verify_player_bind_use_proxy = False
        self.proxy_url = "http://localhost:7890"
        self.user_database_path = None
        self.token_name = "Tsugu"
        self.bandori_station_token = "ZtV4EX2K9Onb"

        self.use_easy_bg = True
        self.compress = True

        self.ban_gacha_simulate_group_data = []

        self.server_list = {0: "日服", 1: "国际服", 2: "台服", 3: "国服", 4: "韩服"}
        self.server_name_to_index = {
            "日服": "0",
            "国际服": "1",
            "台服": "2",
            "国服": "3",
            "韩服": "4",
            "jp": "0",
            "en": "1",
            "tw": "2",
            "cn": "3",
            "kr": "4",
        }

        self.server_index_to_name = {
            "0": "日服",
            "1": "国际服",
            "2": "台服",
            "3": "国服",
            "4": "韩服",
        }

        self.server_index_to_s_name = {
            "0": "jp",
            "1": "en",
            "2": "tw",
            "3": "cn",
            "4": "kr",
        }

        self.features = {
                "car_number_forwarding": True,
                "change_main_server": True,
                "switch_car_forwarding": True,
                "bind_player": True,
                "change_server_list": True,
                "player_status": True,
                "help": True,
            }

        self.commands = [
            {"api": "cardIllustration", "command_name": ["查插画", "查卡面"]},
            {"api": "player", "command_name": ["查玩家", "查询玩家"]},
            {"api": "gachaSimulate", "command_name": ["抽卡模拟", "卡池模拟"]},
            {"api": "gacha", "command_name": ["查卡池"]},
            {"api": "event", "command_name": ["查活动"]},
            {"api": "song", "command_name": ["查歌曲", "查曲"]},
            {"api": "songMeta", "command_name": ["查询分数表", "查分数表"]},
            {"api": "character", "command_name": ["查角色"]},
            {"api": "chart", "command_name": ["查铺面", "查谱面"]},
            {"api": "ycxAll", "command_name": ["ycxall", "ycx all"]},
            {"api": "ycx", "command_name": ["ycx", "预测线"]},
            {"api": "lsycx", "command_name": ["lsycx"]},
            {"api": "ycm", "command_name": ["ycm", "车来"]},
            {"api": "card", "command_name": ["查卡"]},
        ]

        self.car_config = {
            "car": [
                "q1",
                "q2",
                "q3",
                "q4",
                "Q1",
                "Q2",
                "Q3",
                "Q4",
                "缺1",
                "缺2",
                "缺3",
                "缺4",
                "差1",
                "差2",
                "差3",
                "差4",
                "3火",
                "三火",
                "3把",
                "三把",
                "打满",
                "清火",
                "奇迹",
                "中途",
                "大e",
                "大分e",
                "exi",
                "大分跳",
                "大跳",
                "大a",
                "大s",
                "大分a",
                "大分s",
                "长途",
                "生日车",
                "军训",
                "禁fc",
            ],
            "fake": [
                "114514",
                "野兽",
                "恶臭",
                "1919",
                "下北泽",
                "粪",
                "糞",
                "臭",
                "11451",
                "xiabeize",
                "雀魂",
                "麻将",
                "打牌",
                "maj",
                "麻",
                "[",
                "]",
                "断幺",
                "qq.com",
                "腾讯会议",
                "master",
                "疯狂星期四",
                "离开了我们",
                "日元",
                "av",
                "bv",
            ],
        }
        self.help_doc_dict = {
            "玩家状态": """
查询指定玩家的状态信息。
使用示例：
    玩家状态 : 查询你当前默认服务器的玩家状态
    玩家状态  jp : 查询日服玩家状态
    玩家状态 2 : 查询您的第二个绑定玩家的状态(*此功能需要BOT支持)
        
""",
            "开关车牌转发": """
开启或关闭车牌转发功能，针对个人
使用示例：
    开启车牌转发 : 开启车牌转发功能
    关闭车牌转发 : 关闭车牌转发功能
""",
            "绑定玩家": """
绑定玩家ID到当前账号
使用示例：
    绑定玩家 : 绑定玩家开始绑定玩家流程(*请根据BOT的提示进行操作)
""",
            "解除绑定": """
解除绑定玩家ID
使用示例：
    解除绑定 : 解除绑定玩家开始解绑玩家流程(*请根据BOT的提示进行操作)
""",
            "切换主服务器": """
切换主服务器
使用示例：
    主服务器 jp : 切换主服务器到日服
    国服模式 : 切换主服务器到国服
""",
            "设置默认服务器": """
可以改变查询时的资源优先顺序
设置默认服务器，使用空格分隔服务器列表
使用示例：
    设置默认服务器 国服 日服 : 将国服设置为第一服务器，日服设置为第二服务器
""",
            "查卡面": """
根据卡片ID查询卡片插画
使用示例：
    查卡面 1399 :返回1399号卡牌的插画
""",
            "查询玩家": """
查询指定ID玩家的信息。省略服务器名时，默认从你当前的主服务器查询
使用示例：
    查玩家 10000000 : 查询你当前默认服务器中，玩家ID为10000000的玩家信息
    查玩家 40474621 jp : 查询日服玩家ID为40474621的玩家信息
""",
            "卡池模拟": """
模拟抽卡，如果没有卡池ID的话，卡池为当前活动的卡池
使用示例：
    抽卡模拟:模拟抽卡10次
    抽卡模拟 300 922 :模拟抽卡300次，卡池为922号卡池
""",
            "查卡池": """
根据卡池ID查询卡池信息
""",
            "查活动": """
根据关键词或活动ID查询活动信息
使用示例：
    查活动 177 :返回177号活动的信息
    查活动 绿 tsugu :返回所有属性加成为pure，且活动加成角色中包括羽泽鸫的活动列表
    查活动 >255 :返回所有活动ID大于255的活动列表
    查活动 255-256 :返回所有活动ID在255到256之间的活动列表
    查活动 ppp :匹配到 PPP 乐队的活动信息
""",
            "查曲": """
根据关键词或曲目ID查询曲目信息
使用示例：
    查曲 1 :返回1号曲的信息
    查曲 ag lv27 :返回所有难度为27的ag曲列表
    查曲 1 ex :返回1号曲的expert难度曲目信息
    查曲 滑滑蛋 :匹配到 ふわふわ時間 的曲目信息
""",
            "查分数表": """
查询指定服务器的歌曲分数表，如果没有服务器名的话，服务器为用户的默认服务器
""",
            "查角色": """
根据关键词或角色ID查询角色信息
使用示例：
    查角色 10 :返回10号角色的信息
    查角色 吉他 :返回所有角色模糊搜索标签中包含吉他的角色列表
""",
            "查谱面": """
根据曲目ID与难度查询铺面信息
使用示例：
    查谱面 1 :返回1号曲的所有铺面
    查谱面 1 expert :返回1号曲的expert难度铺面
""",
            "ycxall": """
查询所有档位的预测线，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 

使用示例：
    ycxall :返回默认服务器当前活动所有档位的档线与预测线
    ycxall 177 jp:返回日服177号活动所有档位的档线与预测线
""",
            "ycx": """
查询指定档位的预测线，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 
使用示例：
    ycx 1000 :返回默认服务器当前活动1000档位的档线与预测线
    ycx 1000 177 jp:返回日服177号活动1000档位的档线与预测线
""",
            "lsycx": """
与ycx的区别是，lsycx会返回与最近的4期活动类型相同的活动的档线数据
查询指定档位的预测线，与最近的4期活动类型相同的活动的档线数据，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 

使用示例：
    lsycx 1000 :返回默认服务器当前活动的档线与预测线，与最近的4期活动类型相同的活动的档线数据
    lsycx 1000 177 jp:返回日服177号活动1000档位档线与最近的4期活动类型相同的活动的档线数据
""",
            "ycm": """
获取所有车牌车牌
使用示例：
    ycm : 获取所有车牌
""",
            "查卡": """
根据关键词或卡牌ID查询卡片信息，请使用空格隔开所有参数
使用示例：
    查卡 1399 :返回1399号卡牌的信息
    查卡 绿 tsugu :返回所有属性为pure的羽泽鸫的卡牌列表
    查卡 kfes ars :返回所有为kfes的ars的卡牌列表
""",
        }

    def show_docs(self):

        pass
        return None

    def output_config_json(self, path="./config.json"):
        config_pre_json = {
            "backend": "http://tsugubot.com:8080",
            "user_data_backend": "http://tsugubot.com:8080",
            "backend_use_proxy": False,
            "user_data_backend_use_proxy": False,
            "submit_car_number_use_proxy": False,
            "verify_player_bind_use_proxy": False,
            "proxy_url": "http://localhost:7890",
            "user_database_path": None,
            "token_name": "Tsugu",
            "bandori_station_token": "ZtV4EX2K9Onb",
            "use_easy_bg": True,
            "compress": True,
            "ban_gacha_simulate_group_data": [],
            "server_list": {
                "0": "日服",
                "1": "国际服",
                "2": "台服",
                "3": "国服",
                "4": "韩服",
            },
            "server_name_to_index": {
                "日服": "0",
                "国际服": "1",
                "台服": "2",
                "国服": "3",
                "韩服": "4",
                "jp": "0",
                "en": "1",
                "tw": "2",
                "cn": "3",
                "kr": "4",
            },
            "server_index_to_name": {
                "0": "日服",
                "1": "国际服",
                "2": "台服",
                "3": "国服",
                "4": "韩服",
            },
            "server_index_to_s_name": {
                "0": "jp",
                "1": "en",
                "2": "tw",
                "3": "cn",
                "4": "kr",
            },
            "features": {
                "car_number_forwarding": True,
                "change_main_server": True,
                "switch_car_forwarding": True,
                "bind_player": True,
                "change_server_list": True,
                "player_status": True,
                "help": True,
            },
            "commands": [
                {"api": "cardIllustration", "command_name": ["查插画", "查卡面"]},
                {"api": "player", "command_name": ["查玩家", "查询玩家"]},
                {"api": "gachaSimulate", "command_name": ["抽卡模拟", "卡池模拟"]},
                {"api": "gacha", "command_name": ["查卡池"]},
                {"api": "event", "command_name": ["查活动"]},
                {"api": "song", "command_name": ["查歌曲", "查曲"]},
                {"api": "songMeta", "command_name": ["查询分数表", "查分数表"]},
                {"api": "character", "command_name": ["查角色"]},
                {"api": "chart", "command_name": ["查铺面", "查谱面"]},
                {"api": "ycxAll", "command_name": ["ycxall", "ycx all"]},
                {"api": "ycx", "command_name": ["ycx", "预测线"]},
                {"api": "lsycx", "command_name": ["lsycx"]},
                {"api": "ycm", "command_name": ["ycm", "车来"]},
                {"api": "card", "command_name": ["查卡"]},
            ],
            "car_config": {
                "car": [
                    "q1",
                    "q2",
                    "q3",
                    "q4",
                    "Q1",
                    "Q2",
                    "Q3",
                    "Q4",
                    "缺1",
                    "缺2",
                    "缺3",
                    "缺4",
                    "差1",
                    "差2",
                    "差3",
                    "差4",
                    "3火",
                    "三火",
                    "3把",
                    "三把",
                    "打满",
                    "清火",
                    "奇迹",
                    "中途",
                    "大e",
                    "大分e",
                    "exi",
                    "大分跳",
                    "大跳",
                    "大a",
                    "大s",
                    "大分a",
                    "大分s",
                    "长途",
                    "生日车",
                    "军训",
                    "禁fc",
                ],
                "fake": [
                    "114514",
                    "野兽",
                    "恶臭",
                    "1919",
                    "下北泽",
                    "粪",
                    "糞",
                    "臭",
                    "11451",
                    "xiabeize",
                    "雀魂",
                    "麻将",
                    "打牌",
                    "maj",
                    "麻",
                    "[",
                    "]",
                    "断幺",
                    "qq.com",
                    "腾讯会议",
                    "master",
                    "疯狂星期四",
                    "离开了我们",
                    "日元",
                    "av",
                    "bv",
                ],
            },
            "help_doc_dict": {
                "玩家状态": """
查询指定玩家的状态信息。
使用示例：
    玩家状态 : 查询你当前默认服务器的玩家状态
    玩家状态  jp : 查询日服玩家状态
    玩家状态 2 : 查询您的第二个绑定玩家的状态(*此功能需要BOT支持)
        
""",
                "开关车牌转发": """
开启或关闭车牌转发功能，针对个人
使用示例：
    开启车牌转发 : 开启车牌转发功能
    关闭车牌转发 : 关闭车牌转发功能
""",
                "绑定玩家": """
绑定玩家ID到当前账号
使用示例：
    绑定玩家 : 绑定玩家开始绑定玩家流程(*请根据BOT的提示进行操作)
""",
                "解除绑定": """
解除绑定玩家ID
使用示例：
    解除绑定 : 解除绑定玩家开始解绑玩家流程(*请根据BOT的提示进行操作)
""",
                "切换主服务器": """
切换主服务器
使用示例：
    主服务器 jp : 切换主服务器到日服
    国服模式 : 切换主服务器到国服
""",
                "设置默认服务器": """
可以改变查询时的资源优先顺序
设置默认服务器，使用空格分隔服务器列表
使用示例：
    设置默认服务器 国服 日服 : 将国服设置为第一服务器，日服设置为第二服务器
""",
                "查卡面": """
根据卡片ID查询卡片插画
使用示例：
    查卡面 1399 :返回1399号卡牌的插画
""",
                "查询玩家": """
查询指定ID玩家的信息。省略服务器名时，默认从你当前的主服务器查询
使用示例：
    查玩家 10000000 : 查询你当前默认服务器中，玩家ID为10000000的玩家信息
    查玩家 40474621 jp : 查询日服玩家ID为40474621的玩家信息
""",
                "卡池模拟": """
模拟抽卡，如果没有卡池ID的话，卡池为当前活动的卡池
使用示例：
    抽卡模拟:模拟抽卡10次
    抽卡模拟 300 922 :模拟抽卡300次，卡池为922号卡池
""",
                "查卡池": """
根据卡池ID查询卡池信息
""",
                "查活动": """
根据关键词或活动ID查询活动信息
使用示例：
    查活动 177 :返回177号活动的信息
    查活动 绿 tsugu :返回所有属性加成为pure，且活动加成角色中包括羽泽鸫的活动列表
    查活动 >255 :返回所有活动ID大于255的活动列表
    查活动 255-256 :返回所有活动ID在255到256之间的活动列表
    查活动 ppp :匹配到 PPP 乐队的活动信息
""",
                "查曲": """
根据关键词或曲目ID查询曲目信息
使用示例：
    查曲 1 :返回1号曲的信息
    查曲 ag lv27 :返回所有难度为27的ag曲列表
    查曲 1 ex :返回1号曲的expert难度曲目信息
    查曲 滑滑蛋 :匹配到 ふわふわ時間 的曲目信息
""",
                "查分数表": """
查询指定服务器的歌曲分数表，如果没有服务器名的话，服务器为用户的默认服务器
""",
                "查角色": """
根据关键词或角色ID查询角色信息
使用示例：
    查角色 10 :返回10号角色的信息
    查角色 吉他 :返回所有角色模糊搜索标签中包含吉他的角色列表
""",
                "查谱面": """
根据曲目ID与难度查询铺面信息
使用示例：
    查谱面 1 :返回1号曲的所有铺面
    查谱面 1 expert :返回1号曲的expert难度铺面
""",
                "ycxall": """
查询所有档位的预测线，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 

使用示例：
    ycxall :返回默认服务器当前活动所有档位的档线与预测线
    ycxall 177 jp:返回日服177号活动所有档位的档线与预测线
""",
                "ycx": """
查询指定档位的预测线，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 
使用示例：
    ycx 1000 :返回默认服务器当前活动1000档位的档线与预测线
    ycx 1000 177 jp:返回日服177号活动1000档位的档线与预测线
""",
                "lsycx": """
与ycx的区别是，lsycx会返回与最近的4期活动类型相同的活动的档线数据
查询指定档位的预测线，与最近的4期活动类型相同的活动的档线数据，如果没有服务器名的话，服务器为用户的默认服务器。如果没有活动ID的话，活动为当前活动
可用档线:
20, 30, 40, 50, 100, 200, 300, 400, 500, 1000, 2000, 5000, 10000, 20000, 30000, 50000, 

使用示例：
    lsycx 1000 :返回默认服务器当前活动的档线与预测线，与最近的4期活动类型相同的活动的档线数据
    lsycx 1000 177 jp:返回日服177号活动1000档位档线与最近的4期活动类型相同的活动的档线数据
""",
                "ycm": """
获取所有车牌车牌
使用示例：
    ycm : 获取所有车牌
""",
                "查卡": """
根据关键词或卡牌ID查询卡片信息，请使用空格隔开所有参数
使用示例：
    查卡 1399 :返回1399号卡牌的信息
    查卡 绿 tsugu :返回所有属性为pure的羽泽鸫的卡牌列表
    查卡 kfes ars :返回所有为kfes的ars的卡牌列表
""",
            },
        }
        with open(path, "w", encoding="utf-8") as f:
            json.dump(config_pre_json, f, ensure_ascii=False, indent=4)
        print(f"Config data has been output to '{path}'.")

    def reload_from_json(self, path):
        """Reloads configuration data from a JSON string or a JSON file path."""
        with open(path, "r", encoding="utf-8") as file:
            config_data = json.load(file)
        # Iterate over all keys in the input JSON and update the config attributes
        for key, value in config_data.items():
            if hasattr(self, key):
                setattr(self, key, value)
            else:
                print(f"Warning: {key} is not a recognized configuration attribute.")


config = Config()
